using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Citizen : MonoBehaviour 
{
	public GameObject heartPrefab;
	
	public List<Desire> desires;
	public List<DAttribute> attributes;
	
	public Tile tile;
	public int happiness;
	
	public string citizenName;
	public Color colour;
	
	public RagePixelSprite sprite;
	
	public Transform pt1;
	public Transform pt2;
	public Transform pt3;
	
	public GameObject h1;
	public GameObject h2;
	public GameObject h3;
	
	void Awake()
	{
		pt1 = transform.FindChild("Pt1");
		pt2 = transform.FindChild("Pt2");
		pt3 = transform.FindChild("Pt3");
		
		sprite = GetComponent<RagePixelSprite>();
	}
	
	void Start()
	{	
		if (desires == null)
		{
			Debug.Log("creating desires in constructor");
			desires = new List<Desire>();
		}
		attributes = new List<DAttribute>();
		
		//PopulateDesires();
		PopulateAttributes();
	}
	
	public void UpdateDesires(List<DAttribute> attr, bool ownTile)
	{
		if (attr == null) return;
		
		//Debug.Log("Do UpdateDesires in Citizen " + citizenName);
		
		for (int i = 0; i < desires.Count; i++)
		{
			for (int a = 0; a < attr.Count; a++)
			{
				desires[i].ApplyAttribute(attr[a], ownTile);
			}
		}
		
	}
	
	public Desire AddDesire(Desire d)
	{
		if (desires == null)
		{
			Debug.Log("creating desires in add desire");
			desires = new List<Desire>();
		}
		
		desires.Add(d);
		
		return d;
	}
	
	// reset status of all desires
	public void DesireUpdateBegin()
	{
		for (int i = 0; i < desires.Count; i++)
		{
			desires[i].DesireUpdateBegin();
		}
	}
	
	public bool DesiresChanged()
	{
		bool result = false;
		
		for (int i = 0; i < desires.Count; i++)
		{
			bool r = desires[i].DesireChanged();
			
			if (r && desires[i].fulfilled)
			{
				GameObject go = null;
				//Debug.Log("should show heart" + citizenName);
				
				if (i == 0 && h1 == null)
				{
					Debug.Log("heart 1");
					h1 = go = (GameObject)Instantiate(heartPrefab, pt1.position, Quaternion.identity); 
				}
				else if (i == 1 && h2 == null) h2 = go = (GameObject)Instantiate(heartPrefab, pt2.position, Quaternion.identity);
				else if (i == 2 && h3 == null) h3 = go = (GameObject)Instantiate(heartPrefab, pt3.position, Quaternion.identity);
				
				if (go)
				{
					go.SendMessage("BoingEffect");
					go.transform.parent = transform;
				}
			}
			
			result = result || r;
		}
		
		return result;
	}
	
	public void PopulateAttributes()
	{
		DAttribute at = new DAttribute();
		at.type = DAttribute.AttrType.Person;
		at.name = names[Random.Range(0, names.Length)];
		at.color = Citizen.RandomColour();
		citizenName = at.name;
		colour = at.color;
		
		sprite.SetTintColor(at.color);

		attributes.Add(at);
	}
	
	public void PopulateDesires()
	{
		for (int i = 0; i < 3; i++)
		{
			desires.Add(DesireGenerator.Generate());
		}
	}

	void OnPressDown()
	{
		if (tile) return;
		
		Level.Instance.ClearSelected();
		InfoHUD.Instance.DisplayCitizen(gameObject, this);
	}
	
	void OnHover()
	{
		Level.Instance.ClearSelected();
		InfoHUD.Instance.DisplayCitizen(gameObject, this);
	}
	
	public static string [] names = 
	{
		"Eric",
		"Chevy",
		"Crislana",
		"Cody",
		"Lanz",
		"Shane",
		"Karl",
		"Samantha"
	};
	
	public static string GetRandomUniqueName(string first, string second)
	{
		int total = names.Length;
		if (first != null) total--;
		if (second != null) total--;
		
		string [] tempNames = new string[total];
		int pos = 0;
		
		for (int i = 0; i < total; i++)
		{
			bool pass = true;
			if (first != null && first == names[i]) pass = false;
			if (second != null && second == names[i]) pass = false;
			
			if (pass)
			{
				tempNames[pos] = names[i];
				pos++;
			}
		}
		
		return tempNames[Random.Range(0, total)];
	}
	
	public static Color red = new Color(206f / 255f, 81f / 255f, 81f / 255f, 1f);
	public static Color blue =  new Color(120f / 255f, 142f / 255f, 1f, 1f);
	public static Color yellow = new Color(247f / 255f, 203f / 255f, 79f / 255f, 1f);
	public static Color green = new Color(93f / 255f, 200f / 255f, 92f / 255f, 1f);
	
	public static Color RandomColour()
	{
		switch(Random.Range(0,4))
		{
		case 0:
			return red;
		case 1:
			return blue;
		case 2:
			return yellow;
		case 3:
			return green;
		}
		
		return Color.white; // should never be reached
	}
	
	public static string ColourName(Color c)
	{
		if (c == red) return "red";
		if (c == blue) return "blue";
		if (c == yellow) return "yellow";
		if (c == green) return "green";
		
		Debug.Log("error: " + c);
		return "error";
	}
}
