using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class InfoHUD : MonoBehaviour 
{
	public static InfoHUD Instance;
	
	public GUIStyle regularStyle;
	public GUIStyle boldStyle;
	public GUIStyle completedStyle;
	public GUIStyle compactStyle;
	
	public Vector2 desiredResolution;
	public int desiredTextSize;
	
	float s;
	
	GameObject displayedObject;
	Citizen citizen;
	Tile tile;
	
	Tile localTile;
	Citizen localCitizen;
	GameObject localHeart1;
	GameObject localHeart2;
	GameObject localHeart3;
	
	public GameObject heartPrefab;
	
	public Transform tilePoint;
	
	void Awake()
	{
		Instance = this;
		
		s = Screen.width / desiredResolution.x;
		
		textThings = new List<TextInfo>();
		
		regularStyle.fontSize = (int)(desiredTextSize * s);
		boldStyle.fontSize = (int)(desiredTextSize * s);
		completedStyle.fontSize = (int)(desiredTextSize * s);
	}
	
	void Start () 
	{
		localCitizen = transform.FindChild("Citizen").GetComponent<Citizen>();
		localTile = transform.FindChild("Tile").GetComponent<Tile>();
		
		localTile.transform.position = tilePoint.position;
		localCitizen.transform.position = localTile.citizenOffset.position - Vector3.forward * 0.02f;
		
		localHeart1 = (GameObject)Instantiate(heartPrefab, localCitizen.pt1.position - Vector3.forward * 0.01f, Quaternion.identity);
		localHeart2 = (GameObject)Instantiate(heartPrefab, localCitizen.pt2.position - Vector3.forward * 0.01f, Quaternion.identity);
		localHeart3 = (GameObject)Instantiate(heartPrefab, localCitizen.pt3.position - Vector3.forward * 0.01f, Quaternion.identity);
		
		ClearDisplay();
	}
	
	public int score = 0;
	
	void OnGUI()
	{	
		//int indent = 8;
		//GUI.Label(RRect(44, 30, 200, 200), "Tile", boldStyle);
		//GUI.Label(RRect(44 + indent, 40 + desiredTextSize + 4, 200, 200), "Grass tile", regularStyle);
		
		if (!Level.Instance.SpaceForCitizens())
		{
			GUI.Label(RRect(253, 800 - 100, 200, 200), "Score: " + score, boldStyle);
			GUI.Label(RRect(253, 800 - 80, 200, 200), "Play again?", regularStyle);
		}

		if (citizen)
		{
			DrawCitizenInfo();
		}
		
		if (tile)
		{
			if (tile.citizen) DrawCitizenInfo();
			else
			{
				int x = 184 - 30;
				int y = 30;
		
				GUI.Label(RRect(x, y, 500, 500), tile.description, boldStyle);
			}
				
		}
		
		if (textThings != null)
		{
			for (int i = 0; i < textThings.Count; i++)
			{
				
				Vector3 pt = Camera.main.WorldToScreenPoint(textThings[i].trans.position);
				
				//Debug.Log(textThings[i].trans.position + " :: " + pt);
	
				GUI.Label(RRect((int)(pt.x), Screen.height - (int)(pt.y), 200, 200), textThings[i].text, compactStyle);
			}
		}
	}
	
	void DrawCitizenInfo()
	{
		int x = 184 - 30;
		int y = 30;
		
		GUI.Label(RRect(x, y, 500, 500), citizen.citizenName + "'s Desires", boldStyle);
		
		for (int i = 0; i < citizen.desires.Count; i++)
		{
			y += desiredTextSize;
			
			if (citizen.desires[i].fulfilled)
				GUI.Label(RRect(x, y, 500, 500), citizen.desires[i].GetDescription(), completedStyle);
			else
				GUI.Label(RRect(x, y, 500, 500), citizen.desires[i].GetDescription(), regularStyle);
		}
	}
	
	public Rect RRect(int x, int y, int w, int h)
	{
		return new Rect((int)(s * x), (int)(s * y), (int)(s * w), (int)(s * h));
	}
	
	public int RInt(int val)
	{
		return (int)(val * s);
	}
	
	public float RFloat(float val)
	{
		return val * s;
	}
	
	public void DisplayCitizen(GameObject go, Citizen c)
	{
		citizen = c;
		tile = null;
		
		if (c != null)
		{
			localCitizen.renderer.enabled = true;
			localCitizen.sprite.SetTintColor(c.colour);
			localCitizen.transform.position = tilePoint.position;
		}
	}
	
	
	public class TextInfo
	{
		public string text;
		public Transform trans;
		
		public TextInfo(Transform trans, string text)
		{
			this.text = text;
			this.trans = trans;
		}
	}
	
	List<TextInfo> textThings;
	
	public void AddText(Transform t, string text)
	{
		TextInfo ti = new TextInfo(t, text);
		textThings.Add(ti);
	}
	
	public void DisplayTile(GameObject go, Tile t)
	{
		//if (displayedObject)
		//{
			// cleanup old copy of gfx
		//	Destroy(displayedObject);
		//}
		
		//displayedObject = (GameObject)Instantiate(go, tilePoint.position, Quaternion.identity);
		//displayedObject.collider.enabled = false;
		//Destroy(displayedObject.GetComponent<iTween>());
		//if (t.citizen) iTween.Stop(displayedObject.transform.FindChild("Citizen(Clone)").gameObject);
		ClearDisplay();
		
		localTile.renderer.enabled = true;
		localTile.sprite.SetSprite(t.spriteName);
			
		if (t.citizen)
		{
			localCitizen.renderer.enabled = true;
			localCitizen.sprite.SetTintColor(t.citizen.colour);
			localCitizen.transform.position = localTile.citizenOffset.position;
			if (t.citizen.h1) localHeart1.renderer.enabled = true;
			if (t.citizen.h2) localHeart2.renderer.enabled = true;
			if (t.citizen.h3) localHeart3.renderer.enabled = true;
		}
		
		//localTile.sprite.SetSprite(t.spriteName
		
		tile = t;
		citizen = t.citizen;
	}
	
	public void ClearDisplay()
	{
		//if (displayedObject)
		//{
		//	// cleanup old copy of gfx
		//	Destroy(displayedObject);
		//}
		
		localCitizen.renderer.enabled = false;
		localTile.renderer.enabled = false;
		localHeart1.renderer.enabled = false;
		localHeart2.renderer.enabled = false;
		localHeart3.renderer.enabled = false;
		
		//displayedObject = null;
		tile = null;
		citizen = null;
	}
}
