using UnityEngine;
using System.Collections;

public class InputManager : MonoBehaviour 
{
	bool processMouseInput;
	bool processTouchInput;
	
	public static InputManager Instance;
	
	GameObject downObject;
	GameObject draggedObject;
	
	void Start()
	{
		Instance = this;
		
		processMouseInput = true;
		processTouchInput = true;
	}
	
	
	void Update () 
	{
		if (processMouseInput) UpdateMouse();
		if (processTouchInput) UpdateTouch();
	}
	
	void UpdateMouse()
	{
		if (Input.GetMouseButtonDown(0))
		{
			downObject = Pickup(RaycastSendMessage("OnPressDown", Input.mousePosition.x, Input.mousePosition.y));
			
		}
		
		if (Input.GetMouseButtonUp(0))
		{
			if (RaycastGetObject(Input.mousePosition.x, Input.mousePosition.y) == downObject)
				RaycastSendMessage("OnPressUp", Input.mousePosition.x, Input.mousePosition.y);
			
			Drop(RaycastGetObject(Input.mousePosition.x, Input.mousePosition.y));
		}
		
		if (Input.GetMouseButton(0) && draggedObject)
		{
			Vector3 pos = Camera.main.ScreenToWorldPoint(new Vector3(Input.mousePosition.x, Input.mousePosition.y, 0));
			pos.z = draggedObject.transform.position.z;
			
			draggedObject.transform.position = pos;
			
		}
		
		if (!draggedObject)// && Input.GetMouseButton(0))
		{
			RaycastSendMessage("OnHover", Input.mousePosition.x, Input.mousePosition.y);
		}
	}
	
	GameObject Pickup(GameObject go)
	{
		if (go != null)
		{
			DragObject dobj = go.GetComponent<DragObject>();
			
			if (dobj)
			{
				dobj.startPosition = go.transform.position;
				dobj.colliderEnabled = go.collider.enabled;
				
				draggedObject = go;
				go.collider.enabled = false;
			}
		}
		
		return go;
	}
	
	void Drop(GameObject go)
	{
		if (draggedObject)
		{
			if (go && go.GetComponent<AcceptDrop>())
			{
				go.SendMessage("OnDrop", draggedObject, SendMessageOptions.DontRequireReceiver);
			}
			else
			{
				DragObject.RejectDrop(draggedObject);
			}
			
			draggedObject = null;
		}
	}
	
	//-------------------------------------------------------------------------
	// touch - single input only
	//-------------------------------------------------------------------------
	
	bool down = false;
	
	void UpdateTouch()
	{
		if (Input.touchCount > 0)
		{
			if (!down)
			{
				
				downObject = Pickup(RaycastSendMessage("OnPressDown", Input.touches[0].position.x, Input.touches[0].position.y));
				down = true;
			}
		}
		
		if (down && Input.touchCount > 0 && Input.touches[0].phase == TouchPhase.Ended)
		{	
			// check if it's the same object
			if (RaycastGetObject(Input.touches[0].position.x, Input.touches[0].position.y) == downObject)
				RaycastSendMessage("OnPressUp", Input.touches[0].position.x, Input.touches[0].position.y);
			
			Drop(RaycastGetObject(Input.touches[0].position.x, Input.touches[0].position.y));
			down = false;
		}
		
		if (Input.touchCount > 0 && !draggedObject)
		{
			RaycastSendMessage("OnHover", Input.touches[0].position.x, Input.touches[0].position.y);
		}
		
		if (down && Input.touchCount == 0)
		{
			// crap we probably missed it
			down = false;
			Drop(null);
			Debug.Log("missed it!");
		}
	}
	
	//-------------------------------------------------------------------------
	// helpers
	//-------------------------------------------------------------------------
	
	GameObject RaycastSendMessage(string message, float x, float y)
	{
		GameObject go = RaycastGetObject(x, y);
		
		if (go != null)
		{
			go.SendMessage(message, SendMessageOptions.DontRequireReceiver);
		}
		
		return go;
	}
	
	GameObject RaycastGetObject(float x, float y)
	{
		Vector3 point = Camera.main.ScreenToWorldPoint(new Vector3(x, y, 0));
		
		RaycastHit hit;
		
		if (Physics.Raycast(point, Vector3.forward, out hit))
		{	
			return hit.transform.gameObject;
		}
		
		return null;
	}

}
