using UnityEngine;
using System.Collections;

public class Level : MonoBehaviour 
{
	public static Level Instance;
	
	public Tile selectedTile;
	
	public int width;
	public int height;
	
	Tile [,] tiles;
	
	public int numCitizens;
	public int numObjects;
	
	public int maxCount;

	void Awake()
	{
		Level.Instance = this;
		
		maxCount = width * height;
		
		GameObject topLeft = transform.FindChild("TopLeftTile").gameObject;
		tiles = new Tile[width, height];
		
		for (int x = 0; x < width; x++)
		{
			for (int y = 0; y < height; y++)
			{
				Tile t = Tile.RaycastTile(topLeft.transform.position + new Vector3(x * Tile.TILE_SIZE, -y * Tile.TILE_SIZE, 0));
				t.spriteName = "Grass";
				tiles[x,y] = t;
			}
		}
	}
	
	
	// temp
	bool once = false;
	void Update()
	{
		if (!once)
		{
			once = true;
		
			CreateRandomLevel();
		}
			
		if (Input.GetKeyDown(KeyCode.A))
		{
			Debug.Log("level cheat");
			numCitizens = maxCount;
		}
	}
	
	public int CalculateScore()
	{
		int total = 0;
		
		for (int x = 0; x < width; x++)
		{
			for (int y = 0; y < height; y++)
			{
				Tile t = tiles[x,y];
				
				if (t.citizen)
				{
					if (t.citizen.h1) total++;
					if (t.citizen.h2) total++;
					if (t.citizen.h3) total++;
				}
			}
		}
		
		return total;
	}
	
	void CreateRandomLevel()
	{
		
		for (int r = 0; r < 12; r++)
		{
			Tile t = tiles[Random.Range(0, width), Random.Range(0, height)];
			t.SetTile(Tile.TileType.Sand);
		}
		
		for (int r = 0; r < 15; r++)
		{
			Tile t = tiles[Random.Range(0, width), Random.Range(0, height)];
			t.SetTile(Tile.TileType.Mountain);
		}
		
		for (int r = 0; r < 6; r++)
		{
			Tile t = tiles[Random.Range(0, width), Random.Range(0, height)];
			t.SetTile(Tile.TileType.Lake);
		}
	}
	
	public void SelectTile(Tile tile)
	{
		if (tile == selectedTile) return;
		
		if (selectedTile)
		{
			selectedTile.renderer.material.SetColor("_Flash", new Color(0f, 0f, 0f, 0f));
			
			//selectedTile.renderer.material = selectedTile.renderer.sharedMaterial;
		}
		
		selectedTile = tile;
		
		InfoHUD.Instance.DisplayTile(tile.gameObject, tile);
		
		tile.renderer.material.SetColor("_Flash", new Color(0.2f, 0.2f, 0.2f, 0f));
	}
	
	public void ClearSelected()
	{
		if (selectedTile)
		{
			selectedTile.renderer.material.SetColor("_Flash", new Color(0f, 0f, 0f, 0f));
			
			//selectedTile.renderer.material = selectedTile.renderer.sharedMaterial;
		}
		selectedTile = null;
		InfoHUD.Instance.ClearDisplay();
	}
	
	public bool SpaceForObjects()
	{
		return numObjects + 1 <=maxCount;
	}
	
	public bool SpaceForCitizens()
	{
		
		return numCitizens + 1<= maxCount;
	}
}
