using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class PersonGenerator : MonoBehaviour 
{
	public GameObject personPrefab;
	
	int easy = 30;
	int medium = 40;
	int hard = 20;
	int rare = 10;
	
	public Citizen Generate(Transform at)
	{
		GameObject go = (GameObject)Instantiate(personPrefab, at.position, Quaternion.identity);
	
		Citizen c = go.GetComponent<Citizen>();
		Level.Instance.numCitizens++;
		
		rand = Random.Range(0, 100);
		lowerBound = 0;

		if (CheckRange(easy))
		{
			Debug.Log("easy person");
			EasyCitizen(c);
		}
		else if (CheckRange(medium))
		{
			Debug.Log("medium desire");
			MediumCitizen(c);
		}
		else if (CheckRange(hard))
		{
			Debug.Log("hard desire");
			HardCitizen(c);
		}	
		else if (CheckRange(rare))
		{
			Debug.Log("rare desire");
			RareCitizen(c);
		}
		else
			Debug.LogError("NO DEISRE TYPE");
		
		return c;
	}

	void EasyCitizen(Citizen c)
	{
		int number = 5;
		int pick = Random.Range(0, number);
		Debug.Log("Easy choice: " + pick);
		switch (pick)
		{
		case 0:
			// 3 same name
			SpecificPersonNameDesire nameDesire = new SpecificPersonNameDesire();
			c.AddDesire(nameDesire);
			c.AddDesire(new SpecificPersonNameDesire(nameDesire.personName));
			c.AddDesire(new SpecificPersonNameDesire(nameDesire.personName));
			break;
		case 1:
			c.AddDesire(new SpecificPersonNameDesire());
			c.AddDesire(new AnyColourDesire(2,4));
			c.AddDesire(new ContentDesire());
			break;
		case 2:
			c.AddDesire(new PlaceNearNameDesire());
			c.AddDesire(new PersonDesire(3, 4));
			c.AddDesire(new ContentDesire());
			break;
		case 3:
			c.AddDesire(new PlaceAtNameDesire());
			c.AddDesire(new PersonColourDesire());
			c.AddDesire(new AnyColourDesire(1,1));
			break;
		case 4:
			c.AddDesire(new PersonColourDesire(1, 1));
			c.AddDesire(new AnyColourDesire(1, 1));
			c.AddDesire(new AnyColourDesire(1, 1));
			break;
		}
	}

	void MediumCitizen(Citizen c)
	{
		int number = 5;
		int pick = Random.Range(0, number);
		Debug.Log("Medium choice: " + pick);
		switch (pick)
		{
		case 0:
			// 2 different people and one colour
			SpecificPersonNameDesire sc = (SpecificPersonNameDesire)c.AddDesire(new SpecificPersonNameDesire());
			c.AddDesire(new SpecificPersonNameDesire(Citizen.GetRandomUniqueName(sc.personName, null)));
			c.AddDesire(new AnyColourDesire(1, 1));
			break;
		case 1:
			// 3 of the same colour at a specific spot
			c.AddDesire(new PlaceAtNameDesire());
			c.AddDesire(new PersonColourDesire(3, 3));
			c.AddDesire(new PersonDesire());
			break;
		case 2:
			c.AddDesire(new PlaceNearNameDesire());
			c.AddDesire(new PersonDesire());
			c.AddDesire(new AnyColourDesire(1, 1));
			break;
		case 3:
			c.AddDesire(new PersonColourDesire());
			c.AddDesire(new PersonColourDesire(1, 1));
			c.AddDesire(new PlaceNearNameDesire());
			break;
		case 4:
			c.AddDesire(new SpecificPersonNameDesire());
			c.AddDesire(new PersonColourDesire(2,2));
			c.AddDesire(new PersonDesire(4,4));
			break;
		}
	}
	
	void HardCitizen(Citizen c)
	{
		int number = 5; // <------------ update if needed
		int pick = Random.Range(0, number);
		Debug.Log("Hard choice: " + pick);
		switch (pick)
		{
		case 0:
			// 3 different people
			SpecificPersonNameDesire s1 = new SpecificPersonNameDesire(Citizen.GetRandomUniqueName(null, null));
			SpecificPersonNameDesire s2 = new SpecificPersonNameDesire(Citizen.GetRandomUniqueName(s1.personName, null));
			SpecificPersonNameDesire s3 = new SpecificPersonNameDesire(Citizen.GetRandomUniqueName(s1.personName, s2.personName));
			c.AddDesire(s1);
			c.AddDesire(s2);
			c.AddDesire(s3);
			break;
		case 1:
			c.AddDesire(new PlaceAtNameDesire());
			c.AddDesire(new SpecificPersonNameDesire());
			c.AddDesire(new PersonColourDesire(2,4));
			break;
		case 2:
			c.AddDesire(new AnyColourDesire(2, 2));
			c.AddDesire(new AnyColourDesire(2, 2));
			c.AddDesire(new ContentDesire());
			break;
		case 3:
			// 4 colours 
			Desire dc = c.AddDesire(new AnyColourDesire(1, 3));
			c.AddDesire(new AnyColourDesire(1, 4 - dc.fulfilledAmount));
			c.AddDesire(new PersonDesire());
			break;
		case 4:
			c.AddDesire(new AnyColourDesire(4,4));
			c.AddDesire(new ContentDesire());
			c.AddDesire(new ContentDesire());
			break;
		}
	}
	
	
	void RareCitizen(Citizen c)
	{
		int number = 1; // <------------ update if needed
		int pick = Random.Range(0, number);
		Debug.Log("Rare choice: " + pick);
		switch (pick)
		{
		case 0:
			c.colour = Color.white;
			c.AddDesire(new ContentDesire());
			c.AddDesire(new ContentDesire());
			c.AddDesire(new ContentDesire());
			break;
		case 1:
			break;
		case 2:
			break;
		case 3:
			break;
		case 4:
			break;
		}
	}
	
	
	int rand;
	int lowerBound;
	bool CheckRange(int percent)
	{
		bool result = false;
		
		if (rand >= lowerBound && rand < lowerBound + percent)
		{
			result = true;	
		}
		
		lowerBound += percent;
		
		return result;
	}
	
	
}
